@@ -1,7 +1,8 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2001-2003 Mikael Hallendal <micke@imendio.com>
  * Copyright (C) 2003      CodeFactory AB
+ * Copyright (C) 2008      Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,53 +20,36 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreestore.h>
+#include <gtk/gtk.h>
 
 #include "dh-marshal.h"
 #include "dh-book-tree.h"
 
-#define d(x)
-
 typedef struct {
-        GdkPixbuf *pixbuf_opened;
-        GdkPixbuf *pixbuf_closed;
-        GdkPixbuf *pixbuf_helpdoc;
-} DhBookTreePixbufs;
-
-typedef struct {
-	const gchar *uri;
-	gboolean     found;
-	GtkTreeIter  iter;
-	GtkTreePath *path;
+        const gchar *uri;
+        gboolean     found;
+        GtkTreeIter  iter;
+        GtkTreePath *path;
 } FindURIData;
 
-struct _DhBookTreePriv {
-	GtkTreeStore      *store;
+typedef struct {
+        GtkTreeStore *store;
+        GNode        *link_tree;
+} DhBookTreePriv;
 
-	DhBookTreePixbufs *pixbufs;
- 	GNode             *link_tree;
-};
-
-static void book_tree_class_init           (DhBookTreeClass      *klass);
-static void book_tree_init                 (DhBookTree           *tree);
-static void book_tree_finalize             (GObject              *object);
-static void book_tree_add_columns          (DhBookTree           *tree);
-static void book_tree_setup_selection      (DhBookTree           *tree);
-static void book_tree_populate_tree        (DhBookTree           *tree);
-static void book_tree_insert_node          (DhBookTree           *tree,
-					    GNode                *node,
-					    GtkTreeIter          *parent_iter);
-static void book_tree_create_pixbufs       (DhBookTree           *tree);
-static void book_tree_selection_changed_cb (GtkTreeSelection     *selection,
-					    DhBookTree           *tree);
+static void dh_book_tree_class_init        (DhBookTreeClass  *klass);
+static void dh_book_tree_init              (DhBookTree       *tree);
+static void book_tree_add_columns          (DhBookTree       *tree);
+static void book_tree_setup_selection      (DhBookTree       *tree);
+static void book_tree_populate_tree        (DhBookTree       *tree);
+static void book_tree_insert_node          (DhBookTree       *tree,
+                                            GNode            *node,
+                                            GtkTreeIter      *parent_iter);
+static void book_tree_selection_changed_cb (GtkTreeSelection *selection,
+                                            DhBookTree       *tree);
 
 enum {
         LINK_SELECTED,
@@ -73,115 +57,71 @@ enum {
 };
 
 enum {
-	COL_OPEN_PIXBUF,
-	COL_CLOSED_PIXBUF,
 	COL_TITLE,
 	COL_LINK,
+	COL_WEIGHT,
 	N_COLUMNS
 };
 
-static GtkTreeViewClass *parent_class = NULL;
-static gint              signals[LAST_SIGNAL] = { 0 };
+G_DEFINE_TYPE (DhBookTree, dh_book_tree, GTK_TYPE_TREE_VIEW);
 
-GType
-dh_book_tree_get_type (void)
+#define GET_PRIVATE(instance) G_TYPE_INSTANCE_GET_PRIVATE \
+  (instance, DH_TYPE_BOOK_TREE, DhBookTreePriv);
+
+static gint signals[LAST_SIGNAL] = { 0 };
+
+static void
+book_tree_finalize (GObject *object)
 {
-        static GType type = 0;
+	DhBookTreePriv *priv = GET_PRIVATE (object);
 
-        if (!type) {
-                static const GTypeInfo info = {
-                        sizeof (DhBookTreeClass),
-			NULL,
-			NULL,
-                        (GClassInitFunc) book_tree_class_init,
-			NULL,
-			NULL,
-                        sizeof (DhBookTree),
-			0,
-                        (GInstanceInitFunc) book_tree_init,
-                };
+	g_object_unref (priv->store);
 
-		type = g_type_register_static (GTK_TYPE_TREE_VIEW,
-					       "DhBookTree",
-					       &info, 0);
-        }
-
-        return type;
+        G_OBJECT_CLASS (dh_book_tree_parent_class)->finalize (object);
 }
 
 static void
-book_tree_class_init (DhBookTreeClass *klass)
+dh_book_tree_class_init (DhBookTreeClass *klass)
 {
-        GObjectClass   *object_class;
-
-        object_class = (GObjectClass *) klass;
-        parent_class = g_type_class_peek_parent (klass);
+        GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = book_tree_finalize;
 
         signals[LINK_SELECTED] =
-                g_signal_new ("link_selected",
+                g_signal_new ("link-selected",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (DhBookTreeClass,
-					       link_selected),
+			      0,
 			      NULL, NULL,
-			      dh_marshal_VOID__POINTER,
+			      _dh_marshal_VOID__POINTER,
 			      G_TYPE_NONE,
 			      1, G_TYPE_POINTER);
+
+	g_type_class_add_private (klass, sizeof (DhBookTreePriv));
 }
 
 static void
-book_tree_init (DhBookTree *tree)
+dh_book_tree_init (DhBookTree *tree)
 {
         DhBookTreePriv *priv;
 
-        priv        = g_new0 (DhBookTreePriv, 1);
-	priv->store = gtk_tree_store_new (N_COLUMNS,
-					  GDK_TYPE_PIXBUF,
-					  GDK_TYPE_PIXBUF,
-					  G_TYPE_STRING,
-					  G_TYPE_POINTER);
+        priv = GET_PRIVATE (tree);
 
-        tree->priv = priv;
+	priv->store = gtk_tree_store_new (N_COLUMNS,
+					  G_TYPE_STRING,
+					  G_TYPE_POINTER,
+                                          PANGO_TYPE_WEIGHT);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
 				 GTK_TREE_MODEL (priv->store));
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
 
-	book_tree_create_pixbufs (tree);
-
 	book_tree_add_columns (tree);
 
 	book_tree_setup_selection (tree);
 }
 
-static void
-book_tree_finalize (GObject *object)
-{
-	DhBookTree     *tree;
-	DhBookTreePriv *priv;
-
-	tree = DH_BOOK_TREE (object);
-	priv  = tree->priv;
-
-	g_object_unref (priv->store);
-
-	g_object_unref (priv->pixbufs->pixbuf_opened);
-	g_object_unref (priv->pixbufs->pixbuf_closed);
-	g_object_unref (priv->pixbufs->pixbuf_helpdoc);
-	g_free (priv->pixbufs);
-
-	g_free (priv);
-
-	tree->priv = NULL;
-
-	if (G_OBJECT_CLASS (parent_class)->finalize) {
-		G_OBJECT_CLASS (parent_class)->finalize (object);
-	}
-}
-
 static void
 book_tree_add_columns (DhBookTree *tree)
 {
@@ -190,16 +130,6 @@ book_tree_add_columns (DhBookTree *tree)
 
 	column = gtk_tree_view_column_new ();
 
-	cell = gtk_cell_renderer_pixbuf_new ();
-	gtk_tree_view_column_pack_start (column, cell, FALSE);
-	gtk_tree_view_column_set_attributes (
-		column,
-		cell,
-		"pixbuf", COL_OPEN_PIXBUF,
-		"pixbuf-expander-open", COL_OPEN_PIXBUF,
-		"pixbuf-expander-closed", COL_CLOSED_PIXBUF,
-		NULL);
-
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "ellipsize", PANGO_ELLIPSIZE_END,
@@ -207,6 +137,7 @@ book_tree_add_columns (DhBookTree *tree)
 	gtk_tree_view_column_pack_start (column, cell, TRUE);
 	gtk_tree_view_column_set_attributes (column, cell,
 					     "text", COL_TITLE,
+                                             "weight", COL_WEIGHT,
 					     NULL);
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
@@ -229,14 +160,9 @@ book_tree_setup_selection (DhBookTree *tree)
 static void
 book_tree_populate_tree (DhBookTree *tree)
 {
-        DhBookTreePriv *priv;
+        DhBookTreePriv *priv = GET_PRIVATE (tree);
 	GNode          *node;
 
-        priv = tree->priv;
-
-	/* Use the link tree ... */
-/* 	books = dh_bookshelf_get_books (priv->bookshelf); */
-
 	for (node = g_node_first_child (priv->link_tree);
 	     node;
 	     node = g_node_next_sibling (node)) {
@@ -250,39 +176,27 @@ book_tree_insert_node (DhBookTree  *tree,
 		       GtkTreeIter *parent_iter)
 
 {
-        DhBookTreePriv *priv;
-	GtkTreeIter     iter;
+        DhBookTreePriv *priv = GET_PRIVATE (tree);
 	DhLink         *link;
+	GtkTreeIter     iter;
+        PangoWeight     weight;
 	GNode          *child;
 
-        priv = tree->priv;
 	link = node->data;
 
 	gtk_tree_store_append (priv->store, &iter, parent_iter);
 
-	if (link->type == DH_LINK_TYPE_BOOK) {
-		gtk_tree_store_set (priv->store, &iter,
-				    COL_OPEN_PIXBUF,
-				    priv->pixbufs->pixbuf_opened,
-				    COL_CLOSED_PIXBUF,
-				    priv->pixbufs->pixbuf_closed,
-				    COL_TITLE,
-				    link->name,
-				    COL_LINK,
-				    link,
-				    -1);
+	if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
+                weight = PANGO_WEIGHT_BOLD;
 	} else {
-		gtk_tree_store_set (priv->store, &iter,
-				    COL_OPEN_PIXBUF,
-				    priv->pixbufs->pixbuf_helpdoc,
-				    COL_CLOSED_PIXBUF,
-				    priv->pixbufs->pixbuf_helpdoc,
-				    COL_TITLE,
-				    link->name,
-				    COL_LINK,
-				    link,
-				    -1);
-	}
+                weight = PANGO_WEIGHT_NORMAL;
+        }
+
+        gtk_tree_store_set (priv->store, &iter,
+                            COL_TITLE, dh_link_get_name (link),
+                            COL_LINK, link,
+                            COL_WEIGHT, weight,
+                            -1);
 
 	for (child = g_node_first_child (node);
 	     child;
@@ -291,33 +205,20 @@ book_tree_insert_node (DhBookTree  *tree,
 	}
 }
 
-static void
-book_tree_create_pixbufs (DhBookTree *tree)
-{
- 	DhBookTreePixbufs *pixbufs;
-
-	pixbufs = g_new0 (DhBookTreePixbufs, 1);
-
-	pixbufs->pixbuf_closed = gdk_pixbuf_new_from_file (SHAREDIR "/images/book_closed.png", NULL);
-	pixbufs->pixbuf_opened = gdk_pixbuf_new_from_file (SHAREDIR "/images/book_open.png", NULL);
-	pixbufs->pixbuf_helpdoc = gdk_pixbuf_new_from_file (SHAREDIR "/images/helpdoc.png", NULL);
-
-        tree->priv->pixbufs = pixbufs;
-}
-
 static void
 book_tree_selection_changed_cb (GtkTreeSelection *selection,
 				DhBookTree       *tree)
 {
-	GtkTreeIter  iter;
-	DhLink      *link;
+        DhBookTreePriv *priv = GET_PRIVATE (tree);
+        GtkTreeIter     iter;
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-		gtk_tree_model_get (GTK_TREE_MODEL (tree->priv->store),
-				    &iter, COL_LINK, &link, -1);
-
-		d(g_print ("emiting '%s'\n", link->uri));
+                DhLink *link;
 
+		gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
+				    &iter,
+                                    COL_LINK, &link,
+                                    -1);
 		g_signal_emit (tree, signals[LINK_SELECTED], 0, link);
 	}
 }
@@ -325,12 +226,13 @@ book_tree_selection_changed_cb (GtkTreeSelection *selection,
 GtkWidget *
 dh_book_tree_new (GNode *books)
 {
-        DhBookTree *tree;
+        DhBookTree     *tree;
+        DhBookTreePriv *priv;
 
 	tree = g_object_new (DH_TYPE_BOOK_TREE, NULL);
+        priv = GET_PRIVATE (tree);
 
-        tree->priv->link_tree = books;
-
+        priv->link_tree = books;
         book_tree_populate_tree (tree);
 
         return GTK_WIDGET (tree);
@@ -343,24 +245,19 @@ book_tree_find_uri_foreach (GtkTreeModel *model,
 			    FindURIData  *data)
 {
 	DhLink      *link;
-	const gchar *uri;
+        gchar       *link_uri;
 
 	gtk_tree_model_get (model, iter,
 			    COL_LINK, &link,
 			    -1);
 
-	/* A bit hackish, could be made more generic. */
-	if (g_str_has_prefix (data->uri, "file://")) {
-		uri = data->uri + 7;
-	} else {
-		uri = data->uri;
-	}
-
-	if (g_str_has_prefix (uri, link->uri)) {
+        link_uri = dh_link_get_uri (link);
+	if (g_str_has_prefix (data->uri, link_uri)) {
 		data->found = TRUE;
 		data->iter = *iter;
 		data->path = gtk_tree_path_copy (path);
 	}
+        g_free (link_uri);
 
 	return data->found;
 }
@@ -369,13 +266,14 @@ void
 dh_book_tree_select_uri (DhBookTree  *tree,
 			 const gchar *uri)
 {
+        DhBookTreePriv   *priv = GET_PRIVATE (tree);
 	GtkTreeSelection *selection;
 	FindURIData       data;
 
 	data.found = FALSE;
 	data.uri = uri;
 
-	gtk_tree_model_foreach (GTK_TREE_MODEL (tree->priv->store),
+	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
 				(GtkTreeModelForeachFunc) book_tree_find_uri_foreach,
 				&data);
 
@@ -432,5 +330,5 @@ dh_book_tree_get_selected_book_title (DhBookTree *tree)
 			    COL_LINK, &link,
 			    -1);
 
-	return link->name;
+	return dh_link_get_name (link);
 }
