@@ -30,6 +30,7 @@
 #include "dh-search.h"
 #include "dh-preferences.h"
 #include "dh-base.h"
+#include "dh-util.h"
 
 typedef struct {
         DhKeywordModel *model;
@@ -464,6 +465,25 @@ search_cell_data_func (GtkTreeViewColumn *tree_column,
                       NULL);
 }
 
+static gint
+book_cmp (DhLink **a, DhLink **b)
+{
+        return dh_util_cmp_book (*a, *b);
+}
+
+static gboolean
+search_combo_row_separator_func (GtkTreeModel *model,
+                                 GtkTreeIter  *iter,
+                                 gpointer      data)
+{
+        char *label;
+        char *link;
+
+        gtk_tree_model_get (model, iter, 0, &label, 1, &link, -1);
+
+        return (link == NULL && label == NULL);
+}
+
 static GtkWidget *
 search_combo_create (DhSearch *search,
                      GList    *keywords)
@@ -473,15 +493,25 @@ search_combo_create (DhSearch *search,
         GList           *l;
         GtkWidget       *combo;
         GtkCellRenderer *cell;
+        GArray          *books;
+        int              i, nb_books = 0;
 
         store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
-        gtk_list_store_append (store, &iter); 
+        gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, _("All books"),
                             1, NULL,
                             -1);
 
+        /* Add a separator */
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter,
+                            0, NULL,
+                            1, NULL,
+                            -1);
+
+        books = g_array_sized_new (FALSE, FALSE, sizeof(DhLink*), 50);
         for (l = keywords; l; l = l->next) {
                 DhLink *link = l->data;
 
@@ -489,16 +519,30 @@ search_combo_create (DhSearch *search,
                         continue;
                 }
 
-                gtk_list_store_append (store, &iter); 
+                g_array_append_val (books, link);
+                nb_books++;
+        }
+
+        g_array_sort (books, (GCompareFunc)book_cmp);
+
+        for (i = 0; i < nb_books; i++) {
+                DhLink *link = g_array_index(books, DhLink*, i);
+                gtk_list_store_append (store, &iter);
                 gtk_list_store_set (store, &iter,
                                     0, dh_link_get_name (link),
                                     1, dh_link_get_book_id (link),
                                     -1);
         }
 
+        g_array_free (books, TRUE);
+
         combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
         g_object_unref (store);
 
+        gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (combo),
+                                              search_combo_row_separator_func,
+                                              NULL, NULL);
+
         cell = gtk_cell_renderer_text_new ();
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
                                     cell,
